<!DOCTYPE html>
<html lang="en">

<head>
    <title>{{title}}</title>
    <link rel="stylesheet" href="/static/default.css">
    <link rel="stylesheet" href="/static/styles.css">
    <link rel="icon" type="image/x-icon" href="/static/asset/favicon.ico">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <script src="/static/build/lightweight-charts.js"></script>
</head>

<body>
    <div id="container"></div>
    {% if log_btn %}
        <div id="log-href">
            <a href="/log" target="_blank" rel="noopener noreferrer">log</a>
        </div>
    {% endif %}
    <script src="/static/build/bundle.js"></script>
    <script src="/static/build/pywebview.js"></script>
    <script src="/static/index.js"></script>
    <script src="/static/render/index.js"></script>
</body>

</html>